%PropInc

%copyright Mathias Hoffmann, University of Zurich 2004-2013. 
%part of the MATLAB routines to accompany
%'The Consumption-Income Ratio, Entrepreneurial Risk and the US Stock
%market' (JMCB 2014).
%Please cite the paper, if you use these programs for your own research
%purposes.


%CPYmain.m estimates a cointegrating relationship between 
%Consumption, proprietary, labour and other income and the VECM







global T N L Trend XData h

%load files
[GDPbyExp,text]=xlsread('PropIncCPYDataUpdated','GDPbyExpenditure');GDPbyExp=GDPbyExp';
[PersInc,text]=xlsread('PropIncCPYDataUpdated','PersonalIncome');PersInc=PersInc';
[PCEPrices,text]=xlsread('PropIncCPYDataUpdated','PCEandComponents');PCEPrices=PCEPrices';


%adjust size
T=size(GDPbyExp,2);
GDPbyExp=GDPbyExp(:,1:end);
PersInc=PersInc(:,1:end);
PCEPrices=PCEPrices(:,1:end);



%Define consumption




%C=consumption on nondurable-shoes and clothing+services
C=GDPbyExp(9,:)'+GDPbyExp(14,:)'-GDPbyExp(11,:)';%Non-durables+services - clothing & footwear 




%total consumption
if NonDurablesConsumption==0
    C=GDPbyExp(2,:)';
end;

D=GDPbyExp(2,:)'-C;%Durables Consumption

POP=PersInc(42,:)'/1000;%population %'line 39' but there are three non-numbererd (blank lines) in the Table up to numbered line 39.

TotalInc=PersInc(1,:)';%total personal income 
DispInc=PersInc(26,:)';%Disposable Income only
Taxes=PersInc(25,:)';%Current Taxes 
SocSec=PersInc(24,:)';%and (employee) contributions to gvt. social insurance
EmployerContributions=PersInc(7,:)'+PersInc(8,:)';%Employer contributions to private pension & insurance scheme and to government social security;
Transfers=PersInc(16,:)';%-PersInc(24,:)';

PropInc=PersInc(11,:)'+PersInc(12,:)';;%(non-farm)  proprietary income plus rents ;PersInc(10,:)


%Old Definition:
%LabInc=PersInc(2,:)';
%New LL definition (post 2003)
LabInc=PersInc(2,:)'+EmployerContributions-SocSec;
%see http://faculty.haas.berkeley.edu/lettau/data/Data_Revision_2003.txt:
%After-tax labor income (after BEA revisions 2003)  = wages and salaries + transfer payments + employer contributions for employee pensions and insurance - employee contributions for social insurance - taxes
%  **EMPLOYEE CONTRIBUTIONS FOR SOCIAL INSURANCE = contributions for government social insurance - employer contributions for government social insurance.



AssetInc=PersInc(13,:)';%Includes dividends and interest payments
DividendInc=PersInc(15,:)';
InterestInc=PersInc(14,:)';
RentalInc_CapDepr=PersInc(12,:)';
FarmInc=PersInc(10,:)';


if ApportionDivIncome==0
OtherInc=LabInc+AssetInc-DividendInc*AddDivIncomeToProprietors;%+FarmInc;
PropInc=PropInc+AddDivIncomeToProprietors*DividendInc*DivIncomeInCPY;
else
OtherInc=LabInc+AssetInc-DividendInc.*PF_expandedQ;%+FarmInc;
PropInc=PropInc+DividendInc.*(1-PF_expandedQ);
end;



PropInc=PropInc+(Transfers-Taxes).*PropInc./TotalInc;
LabInc=LabInc+(Transfers-Taxes).*LabInc./TotalInc;
DividendInc=DividendInc+(Transfers-Taxes).*DividendInc./TotalInc;
OtherInc=OtherInc+(Transfers-Taxes).*OtherInc./TotalInc;





%Labour Income defined the Lettau-Ludvigson way
LabIncLL=PersInc(3,:)'+PersInc(16,:)'-PersInc(24,:)';%
LabIncLL=LabIncLL-(LabIncLL./(LabIncLL+AssetInc+PropInc)).*PersInc(25,:)';


PCEPI=PCEPrices(1,:)'/100;%PErsonalConsumptionExpenditure PriceIndex PCEPI


%finished loading the data

%take real per capita values and logarithms...
propinc=log(PropInc./POP./PCEPI);
labinc=log(LabInc./POP./PCEPI);
otherinc=log(OtherInc./POP./PCEPI);
assetinc=log(AssetInc./POP./PCEPI);
dividendinc=log(DividendInc./POP./PCEPI);
dispinc=log(DispInc./POP./PCEPI);
cons=log(C./POP./PCEPI);
durab=log(D./POP./PCEPI);

transfers=log(Transfers./POP./PCEPI);


%colllect it into one vector;
Y=[cons,propinc,otherinc];



Y=adjrange(Y,[1,1947;1,2011],XploreRange,4);

[T,N]=size(Y);



[T,N]=size(Y);

XData=[];
Trend=[]

[TraceTest,TraceCV,MaxTest,MaxCV,SigTrace,SigMax]=joh_test(Y,0,0);

%Coinetgration Test results: commented out. Will be displayed in Table 1 
%disp('Results of Cointegration Test');
% L
% disp(['Trend? - ',num2str(~isempty(Trend))]);
% disp(' Trace Test');
% [TraceTest,TraceCV]
% disp(' MaxEV Test');
% [MaxTest,MaxCV]



%h=1;% number of cointegrating relations. Only needed if CPYmain is run on its own, without initializing parameters in another script
%L=2;%L&L 2000 use L=2, L&L 2004 L=3 (in the level VAR)
%XData=ones(T,1)


[beta,alpha,AR,PI,Omega,epsilon]=joh_proc(Y,XData,h,0,0);
betalong=beta;
beta=beta(1:N,:);


%We estimate it by Dynamic OLS
l=3;

dZ=diff(lag0([Y(:,2:N)],(-l:l)));
CointReg=newols(Y(2+l:T-l,1),[Y(2+l:T-l,2:N),ones(T-2*l-1,1),dZ(l+1:T-1-l,:)]);

%beta0=beta./beta(1,1);
beta0=[1;-CointReg.b(1:N-1,1)];
beta0=beta./beta(1);%Take Johansen estimate by default (more stable in shorter samples).
Z=Y*beta0;%Cointegration error


if FixCIvector==1;    
%This part sets the cointegrating vector for the RVECM specification.  This differs from the beta0 estimated above only 
%if the model is estimated  recursively, but the vector has to be fixed to the value from teh whole sample (such as in the OoS predictions)
                                 
beta0=[1, -0.2686, -0.7708]';%Estimate 1952:1-2010:4, DOLS, when y includes divs                               
beta0=[1, -0.2694, -0.7629]';%Estimate 1952:1-2010:4, Johansen, when y includes divs                               


if DivIncomeInCPY==0
beta0=[1.0000,-0.3046,-0.7446]';
end;
if AddDivIncomeToProprietors==1;
beta0=[1.0000,-0.2967,-0.7173 ]';% Estimate from Johansen-procedure for 1952:1-2010:2 sample, new data set,
                                 %when p includes dividend income 
end;  

if ApportionDivIncome==1
beta0=[1.0000,-0.2613,-0.7621]';%DOLS, 1952:1-2010:4
beta0=[ 1.0000, -0.2570,-0.7563]';%Johansen, 1952:1-2010:4
end;

end;                                





XData=[XData,ones(T,1)];
[AR,PI,D,alpha,epsilon,Omega,B,varB,VECM]=RVECM(Y,XData,beta0);





cpy=Y*beta0-mean(Y*beta0);
cp=Y(:,1)-Y(:,2);
cy=Y(:,1)-Y(:,3);


%figure(2)
%plot([cpy])
%betacdy=[1,-0.2388,-0.6833]';
%cdy=[cons,dividendinc,otherinc]*betacdy;
%cdy=adjrange(cdy,[1,1947;4,2003],XploreRange,4);




%Move on to identifying permanent and transitory components
%get interim multiplier G(1):
AR=AR';
G1=eye(N);
for l=1:L-1;
G1=G1-AR((l-1)*N+1:N*l,:)';
end;



estimated_alpha=alpha;%store reduced-form alpha
alpha_o=null(alpha');
beta_o=null(beta0');

C1=beta_o*inv(alpha_o'*G1*beta_o)*alpha_o';
TransChol=chol(alpha'*inv(Omega)*alpha)';
PermChol=chol(alpha_o'*Omega*alpha_o)';
S=[inv(PermChol)*alpha_o';inv(TransChol)*alpha'*inv(Omega)];
Sinv=inv(S);
PI=alpha*beta0';



%Get permanent components a la Granger 
PY=[Y]*(C1*G1)';
TY=Y*(eye(N)-C1*G1)';

GGP=beta_o*inv(alpha_o'*beta_o)*alpha_o';

GGP=Y*GGP';

GGT=alpha*inv(beta'*alpha)*beta';

GGT=Y*GGT';





%Now do the variance decomposition
% for the levels.
ARLevels=VECM2VAR(AR',PI);
AA=Comp_mat(ARLevels);

SelectPerm=[1,0,0;0,1,0;0,0,0];
SelectTrans=[0,0,0;0,0,0;0,0,1];

eins=zeros(L);eins(1,1)=1;

SIGMA=kron(eins,Sinv*Sinv');
SIGMAP=kron(eins,Sinv*SelectPerm*Sinv');
SIGMAT=kron(eins,Sinv*SelectTrans*Sinv');


mse=zeros(size(AA));
   mseperm=mse;
   msetrans=mse;
   MM=zeros(size(AA));
   

   
for k=1:30;
   
   MM=AA^(k-1);
   mse=mse+MM*SIGMA*MM';
   mseperm=mseperm+MM*SIGMAP'*MM';
   msetrans=msetrans+MM*SIGMAT'*MM';
   
   cummse(:,:,k)=mse(1:N,1:N);
   cummseperm(:,:,k)=mseperm(1:N,1:N);
   cummsetrans(:,:,k)=msetrans(1:N,1:N);
   
   VDLevels(:,k)=diag([(cummsetrans(:,:,k))./cummse(:,:,k)]);
   MSE(:,k)=cummse(:,1,k)./cummse(1,1,k);
end;









    

