function [TraceTest,TraceCV,MaxTest,MaxCV,SigTrace,SigMax,joh_test_results]=joh_test(LevelData,const,dummy);
%function [TraceTest,TraceCV,MaxTest,MaxCV,SigTrace,SigMax]=joh_test(LevelData,const,dummy);
%This function returns the Trace- and Maximum Eigenvalue test statistics as
%calculated by the Johansen procedure.
%Note that the MaxTest tests the null of h cointegrating relationships agains 
%the alternative of h+1 cointegrating relationships.
% In general, the trace tests gives a test of H0: h=r1 against r1<h<=r2.
% In the present procedure, r2=N. It then tests for the sequence of
%nulls r1=1...N.
%Critical values are tabulated according to Hamilton (1995), assuming that an 
%unrestricted constant term is included into the regression.
%SigTrace and SigMax are significance tables: they give a 1 if the respective
%statistics is significant at 90% and a 2 if it is significant at the 95%-level.
%otherwise they return 0. 

global T L N XData Trend

MaxCV=[6.69,12.78,18.96,24.92,30.82,30.82;       %90-percent and
       8.083,14.595,21.279,27.341,33.262,33.262]';%95-percent critical values..

TraceCV=[6.69,15.58,28.43,45.248,65.95;    %90 percent
         8.083,17.84,31.25,48.41,69.97]';  %95 percent

if nargin<2;
const=0;
end;
if nargin<3;
dummy=0;
end;

%-----------------------------------------------------------------------
%adjust critical value
if (const==0)&(dummy==0);
MaxCV=[6.69,12.78,18.96,24.92,30.82,0,0;       %90-percent and
       8.083,14.595,21.279,27.341,33.262,0,0]';%95-percent critical values..

TraceCV=[6.69,15.58,28.43,45.248,65.95,89.37,117.73;    %90 percent
         8.083,17.84,31.25,48.41,69.97,93.92,123.04]';  %95 percent

end;
if (const==1)&(dummy==0);%(table 1* Osterwald-Lenum)
TraceCV=[7.5,17.79,31.88,49.92,71.66; %90-percent
         9.13,19.99,34.8,53.42,75.74]';%95-percent
MaxCV=[7.52,13.75,19.77,25.56,31.66,37.45,43.25; %90-percent (Osterwald-Lenum (1991))
       9.24,15.67,22,28.14,34.4,40.30,46.45]';%95-percent
end;
if (const==0)&(dummy==1);%this gives the critical value for the case
                         %where there is a trend included (not a dummy!!!)
                         %case Table2* in Osterwald-Lenum!
TraceCV=[10.49,22.76,39.06,59.14,83.2,110.42,141.01;%90-percent
         12.25,25.32,42.44,62.99,87.31,114.9,146.76]';%95-percent
MaxCV=[10.49,16.85,23.11,29.12,34.75,40.91,46.32;%90-percent;
       12.25,18.96,25.54,31.46,37.52,43.97,49.42]';%95-percent;        
end;         
if (isempty(Trend)==0)&(const==0)&(dummy==0);%table 2/Osterwald-Lenum
TraceCV=[2.57,16.06,31.42,50.74,73.4,100.14,130.84;%90-percent....
         3.74,18.17,34.55,54.64,77.74,104.94,136.61]';%95-percent
MaxCV=[2.57,14.84,21.53,27.76,33.74,39.5,45.49;
       3.74,16.87,23.78,30.33,36.41,42.48,48.45]'
end;

%----------------------------------------------------
%run Johansen-Procedure to get Residuals from auxiliary regression


[beta, alpha,AR,PI,Omega,epsilon,R0,R1]=joh_proc(LevelData,XData,1,const,dummy);
%----------------------------------------------------
%Calculate Canonical Correlations...
S00=1/T.*R0'*R0;
S11=1/T.*R1'*R1;
S01=1/T.*R0'*R1;
S10=S01';

P=S10*inv(S00)*S01;
[n,t]=size(P);
%----------------------------------------------------
%calculate Eigenvalues and Eigenvectors
[EigVect,EigVal]=geigvec(P,S11,N);
EigVal;

%----------------------------------------------------
%this calculates the test-statistics....
MaxTest=zeros(N-1,1);
TraceTest=MaxTest;
constant=(-(T*N)*log(2*pi)-T*N-log(det(S00)))/2;

for i=0:N-1;
MaxTest(i+1,1)=-T*log(1-EigVal(N-i,1));
end; 
TraceTest=cumsum(MaxTest);
hm=mirrmat(length(TraceTest));
TraceTest=hm*TraceTest;
MaxTest=hm*MaxTest;
Likelihood=constant+sum(TraceTest)/2;
%
%end of tests......
%----------------------------------------------------


%build significance-tables:
TraceCV=mirrmat(length(TraceCV))*TraceCV;
MaxCV=mirrmat(length(MaxCV))*MaxCV;
[r,c]=size(TraceCV);%coincides with size(MaxCV);
TraceCV=TraceCV(r-N+1:r,:);
MaxCV=MaxCV(r-N+1:r,:);

for i=1:N;
 if TraceTest(i,1)>TraceCV(i,2); 
 SigTrace(i,1)=2;
 else
  if TraceTest(i,1)>TraceCV(i,1);
  SigTrace(i,1)=1;
  else
  SigTrace(i,1)=0;
  end;
 end;
 
 if MaxTest(i,1)>MaxCV(i,2); 
 SigMax(i,1)=2;
 else
  if MaxTest(i,1)>MaxCV(i,1);
  SigMax(i,1)=1;
  else
  SigMax(i,1)=0;
  end;
 end;
 
end; 
 

joh_test_results.likelihood=Likelihood;
joh_test_results.max=MaxTest;
joh_test_results.trace=TraceTest;
joh_test_results.max_cv=MaxCV;
joh_test_results.trace_cv=TraceCV;






